
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: PythonLibrary(src/python/twitter/common/zookeeper/BUILD:zookeeper-packaged)

from setuptools import setup

setup(**
{   'description': "Implementations of Twitter's service discovery libraries on top of Kazoo.",
    'install_requires': [   'kazoo==1.3.1',
                            'twitter.common.concurrent==0.1.0',
                            'twitter.common.metrics==0.1.0'],
    'name': 'twitter.common.zookeeper',
    'namespace_packages': [   'gen',
                              'gen.twitter',
                              'gen.twitter.thrift',
                              'twitter',
                              'twitter.common'],
    'package_data': {   },
    'package_dir': {   '': 'src'},
    'packages': [   '.',
                    'gen',
                    'gen.twitter',
                    'gen.twitter.thrift',
                    'gen.twitter.thrift.endpoint',
                    'twitter',
                    'twitter.common',
                    'twitter.common.zookeeper',
                    'twitter.common.zookeeper.group',
                    'twitter.common.zookeeper.serverset'],
    'version': '0.1.0'}
)
